# import.m - Database Import Script
# Author:   Jamie McIntosh
# Revision: Michael Hilfer
# Description: Reads database and creates corresponding ID and PAR matrices
#              Compares similarity between each chromosome and database entry for PAR matrix
#              Ranks the similarity distribution and selects the most similar previous design
#              Copies an interpolation file to the chr storage directory to be used in FLUENT
# 
# Note: Requires 'logfile' which contains all the data

# Copyright 2009-2012  Richard James MacPherson, Grant Ingram, 
# Jamie McIntosh and Michael Hilfer.  
# This file is part of PEW V2.

# PEW is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# PEW is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with PEW.  If not, see <http://www.gnu.org/licenses/>.

#
#####################
# Revision History - Jamie McIntosh
# 1.00 - Creation Point
# 1.01 - Survivors of each generation are copied into the next Generation
#        filestore so that repeat testing is removed and script modified to
#        skip these chromosomes in CFD analysis
# 1.02 - Database size now correctly determined
# 1.03 - Database layout format changed to be more readable
# 1.04 - 1.01 removed - skipping affects mutation
# 1.12 - Modified to work with new Design Michael
#####################
# Known Issues
#
#####################

function import()

## Global Parameters
global npar; global par; global iga; global popsize; global constantshi;
global constantslo; global heighthi; global heightlow; global base_dir;
global keep;
 
## Database Settings
hl = 3; % Header lines

## Root of Database Structure
data_root = '/data/l4cfd/mcbxm15/pew/database/';
chdir(sprintf("%s",data_root)); 

## Establish Database Size
[status,output] = system('wc -l logfile'); % Work out number of lines in database
data_str = split(output," "); % Truncates filename from wc output
file_size = str2num(data_str(1,:));
data_size = file_size-hl;

## Read in ID and PAR matrices
fid = fopen('logfile');
line = 1;
row = 1;
data=[];
txt = fgetl (fid);
do
  if line > hl
    tmp = split(txt,"|"); % Puts database entry into a matrix
    data(row,:) = [ str2num(tmp(1,:)) str2num(tmp(3,:)) ];
    row++;
  endif
  line++;
  txt = fgetl (fid);
until (txt == -1)
fclose(fid);

## Determine which Values Require Alternate Similarity Method
scale=[];
col=1;
for sf=npar/6:npar/6:npar
  scale(1,col)=sf;
  col++;
endfor

## Compare Similarity
printf("Interpolation Points...\n");
for chr = 1:1:popsize
sim=[]; % Initialise similarity matrices
sim_ave=[];
  for row = 1:1:data_size
    diff=[]; % Reset diff matrix
    for col = 1:1:npar
      
      if size(data,2)-1 != npar % Similarity is zero if parameter matrix isn't same size
        sim = 0;
      else
        diff(:,col) = abs(par(chr,col)-data(row,col+1));
        if col != scale % Different similarity eqn used for scaling values
          sim(:,col) = ( ( (constantshi-constantslo) - diff(:,col) )/(constantshi-constantslo) )*100; % Finds similarity for curve data
        else
          sim(:,col) = ( ( (heighthi-heightlow) - diff(:,col) )/(heighthi-heightlow) )*100; % Finds similarity for scaling data
        endif
      endif
    endfor
    sim_ave(row,:) = [ row mean(sim) ];
  endfor 
  sim_ave = sortrows(sim_ave,2);   % Sorts in descending order so use last value as most similar
  ip = sim_ave(size(sim_ave,1),1); % Selects the most similar 'id'
  pc = sim_ave(size(sim_ave,1),2); % Selects the most similar 'similarity'
  if pc == 0
    ip = 1; % New type of analysis performed, no data available so interpolate from planar case (ID=1)
  endif
  sy=sprintf("mkdir -p %sGeneration_%d/chr_%d/",base_dir,iga,chr); % Create location to store fluent ip file
  system(sy);
  sy=sprintf("cp %sip/%d.ip %sGeneration_%d/chr_%d/intdata.ip",data_root,ip,base_dir,iga,chr); % Copy fluent ip file
  system(sy);
  printf("chr_%d: id=%d with similarity %.2f%%\n",chr,ip,pc);
  chdir(sprintf("%s",base_dir));
endfor

endfunction
